from unittest import mock

from dagster_aws.emr.pyspark_step_launcher import EmrPySparkStepLauncher

EVENTS = [object(), object(), object()]


@mock.patch(
    "dagster_aws.emr.emr.EmrJobRunner.is_emr_step_complete", side_effect=[False, False, True]
)
@mock.patch(
    "dagster_aws.emr.pyspark_step_launcher.EmrPySparkStepLauncher.read_events",
    side_effect=[EVENTS[0:1], [], EVENTS[0:3]],
)
def test_wait_for_completion(_mock_is_emr_step_complete, _mock_read_events):
    launcher = EmrPySparkStepLauncher(
        region_name="",
        staging_bucket="",
        staging_prefix="",
        wait_for_logs=False,
        action_on_failure="",
        cluster_id="",
        spark_config={},
        local_pipeline_package_path="",
        deploy_local_pipeline_package=False,
    )
    yielded_events = list(
        launcher.wait_for_completion(mock.MagicMock(), None, None, None, None, check_interval=0)
    )
    assert yielded_events == EVENTS



import io

from dagster_aws.s3 import S3FileCache, S3FileHandle


def test_s3_file_cache_file_not_present(mock_s3_resource, mock_s3_bucket):
    file_store = S3FileCache(
        s3_bucket=mock_s3_bucket.name,
        s3_key="some-key",
        s3_session=mock_s3_resource.meta.client,
        overwrite=False,
    )

    assert not file_store.has_file_object("foo")


def test_s3_file_cache_file_present(mock_s3_resource, mock_s3_bucket):
    file_store = S3FileCache(
        s3_bucket=mock_s3_bucket.name,
        s3_key="some-key",
        s3_session=mock_s3_resource.meta.client,
        overwrite=False,
    )

    assert not file_store.has_file_object("foo")

    file_store.write_binary_data("foo", b"bar")

    assert file_store.has_file_object("foo")


def test_s3_file_cache_correct_handle(mock_s3_resource, mock_s3_bucket):
    file_store = S3FileCache(
        s3_bucket=mock_s3_bucket.name,
        s3_key="some-key",
        s3_session=mock_s3_resource.meta.client,
        overwrite=False,
    )

    assert isinstance(file_store.get_file_handle("foo"), S3FileHandle)


def test_s3_file_cache_write_file_object(mock_s3_resource, mock_s3_bucket):
    file_store = S3FileCache(
        s3_bucket=mock_s3_bucket.name,
        s3_key="some-key",
        s3_session=mock_s3_resource.meta.client,
        overwrite=False,
    )

    stream = io.BytesIO(b"content")
    file_store.write_file_object("foo", stream)
